bash-common-api
===============

Common tools for bash scripts


## bash-common-api.sh

Functions:

* `assert_parameters` : 1st parameter, number of expected parameters. Then inherited list of parameters. Check number of parameters, return an error if not equal to expected size.
* `convert_url_to_safe_filename` : Parameter an url, print to stdout a safe file name base on this URL.
* `exit_if_directory_not_exists` : Parameter a directory, print an error message and exit 1 if directory does not exists.
* `exit_if_file_not_exists` : Parameter a file, print an error message and exit 1 if file does not exists.
* `exit_if_variable_not_define_or_empty` : Parameter variable name, print an error message and exit 1 if variable is empty or not defined.
* `exit_if_variable_not_define` : Parameter variable name, print an error message and exit 1 if variable is not define (regardless empty or not).
* `get_variable_safe` : Parameter variable name, return 0 if variable is 0 and print value on stdout (regardless empty or not).
* `is_variable_define_and_not_empty_safe` : Parameter variable name, return 0 if variable is defined and not empty.
* `is_variable_define_safe` : Parameter variable name, return 0 if variable is define (regardless empty or not).
* `P_param` : get required next parameter,  if not exist call usage and return error typical usage is `param_xxx="$( P_param "$@" )" && shift || exit $?`
* `P` : get required next parameter, typical usage is `param_xxx="$( P "$@" )" && shift`
* `trim` : trim each lines of stdin (no parameters)


## message-api.sh

When `MESSAGE_API_HIDE_DATE` is set to `true`, date is not added in messages.

Functions for warnings and errors:

* `echoerror` : `echoerror ...` - Print formatted message to **stderr** using **red** color.
* `echofatal` : `echofatal ...` - Print formatted message to **stderr** using **light red** color.
* `echowarn` : `echowarn  ...` - Print formatted message to **stderr** using **cyan** color.

Functions for debugging:

* `echodebug` : `echodebug  ...` - Print formatted message to **stderr** using **light magenta** color, but only if `MESSAGE_API_SHOW_DEBUG` is set to `true`.

Functions to inform user:

* `echoinfo` : `echoinfo  ...` - Print formatted message using **yellow** color. Use **stdout** if `MESSAGE_API_INFO_STDOUT` is set to `true`
* `echoinfo_quiet` : `echoinfo_quiet  ...` - Use `echoinfo` if `MESSAGE_API_QUIET` is set to `true` otherwise print nothing.
* `echookay` : `echookay  ...` - Print formatted message using **light green** color. Use **stdout** if `MESSAGE_API_OKAY_STDOUT` is set to `true`
* `echookay_quiet` : `echookay_quiet  ...` - Use `echookay` if `MESSAGE_API_QUIET` is set to `true` otherwise print nothing.

## lock-api.sh

Require `LOCKFILE` to be define before sourcing, typical definition is:

```bash
declare -r LOCKFILE='/var/lock/project-name'
```

Functions:

* `lock_api_check_screen` : (no parameter) - return 1 if `screen` is not a parent process of current process.
* `screen_error_display` : (no parameter) - just sleep 15 seconds, if parent process is screen.

TODOC: need more documentation


## json-api.sh

Functions:

* `get_expanded_value_from_json` : `get_expanded_value_from_json JSON_STR KEY`
* `get_optional_expanded_value_from_json` : `get_optional_expanded_value_from_json JSON_STR KEY`
* `is_null_or_empty` : `is_null_or_empty VALUE`
* `load_vars_from_json_file` : `load_vars_from_json JSON_FILE JSON_PATH`
* `load_vars_from_json_string` : `load_vars_from_json_string JSON_STR`
