#!/bin/bash
#
# bash-common-api:json-api.sh
#
if [ "$( type -t echofatal )" != 'function' ] ; then
   # shellcheck disable=SC1091
  source "$( dirname "${BASH_SOURCE[0]}" )/message-api.sh" || exit $?
fi

function load_vars_from_json_file {
  local -r json_file="$1"
  local -r json_path="$2"

  if [ ! -f "${json_file}" ] ; then
    echofatal "${FUNCNAME[0]}() - Can't find file: '${json_file}'."
    return 100
  fi

  local json=
  json="$( jq -r "${json_path}" < "${json_file}" )" || return $?
  local -r json

  if [ "${json}" = 'null' ] ; then
    echofatal "${FUNCNAME[0]}() - Can't find JSON path: '${json_path}'."
    return 100
  fi

  load_vars_from_json_string "${json}" || return $?
}

function load_vars_from_json_string {
  local -r json="$1"

  local keys=
  keys="$( jq -re 'keys[]' <<<"${json}" )" || return $?
  local -r keys

  local key=
  local value=
  while read -r key ; do
    if [ -n "${key}" ] ; then
      echoinfo_quiet "Discover '${key}'" >&2
      value="$( jq -re ".[\"${key}\"]" <<<"${json}" )" || return $?

      declare -gr "${key}"="$( envsubst <<<"${value}" )"
      # shellcheck disable=SC2163
      export "${key}" # required by envsubst
    fi
  done <<<"${keys}"
}

function load_vars_from_json { # DEPRECATED 2021-10-04
  local -r json_file="$1"
  local -r json_path="$2"

  echowarn "${FUNCNAME[0]}() is deprecated, user load_vars_from_file() instead"

  if [ ! -f "${json_file}" ] ; then
    echofatal "${FUNCNAME[0]}() - Can't find file: '${json_file}'."
    return 1
  fi

  local keys=
  keys="$( jq -re "${json_path} | keys[]" < "${json_file}" )" || return $?

  local key=
  local value=
  while read -r key ; do
    if [ -n "${key}" ] ; then
      echoinfo "Set '${key}'"
      value="$( jq -re "${json_path}.${key}" < "${json_file}" )" || return $?

      declare -gr "${key}"="${value}"
      # shellcheck disable=SC2163
      export "${key}" # required by envsubst
    fi
  done <<<"${keys}"
}

function get_expanded_value_from_json {
  local -r json_str="$1"
  local -r key="$2"

  local value=
  value="$( jq -re "${key}" <<<"${json_str}" )" || return $?

  envsubst <<<"${value}" || return $?
}

function get_optional_expanded_value_from_json {
  local -r json_str="$1"
  local -r key="$2"

  local value=
  value="$( jq -r "${key}" <<<"${json_str}" )" || return $?

  envsubst <<<"${value}" || return $?
}

function is_null_or_empty {
  local -r value="$1"

  if [ "${value}" = 'null' ] ; then
    return 0
  elif [ -z "${value}" ] ; then
    return 0
  fi

  return 1
}
