#!/bin/bash
#
# freebox-tools:bin/api/freebox-network-api.sh
#

function freebox_get_parental_filter_json {
  local -i id="$1"

  freebox_api_get_with_result_if_success "/parental/filter/${id}" || return $?
}

function freebox_get_parental_filters_json {
  freebox_api_get_with_result_if_success '/parental/filter/' || return $?
}

function freebox_get_parental_config_json {
  freebox_api_get_with_result_if_success '/parental/config/' || return $?
}

function freebox_network_control_json {
  local -r profile_id="$1"

  if [ -z "${profile_id}" ] || [ "${profile_id}" = '0' ] ; then
    freebox_api_get_with_result_if_success "/network_control" | jq_normalize
  else
    freebox_api_get_with_result_if_success "/network_control/${profile_id}" | jq_normalize
  fi
}

function __network_control_get_profile_id_for_mac_address__ {
  local -r mac_addr_upper_case="$1"

  local result=
  result="$( freebox_network_control_json '' )" || return $?
  result="$(
    jq -c --arg id "${mac_addr_upper_case}" \
        '.[] | select( .hosts[].l2ident.id == $id) | .profile_id' <<<"${result}"
    )" || return $?

  if [ -z "${result}" ] ; then
    echofatal "Pas d'adresse mac trouvée: '${mac_addr_upper_case}' (essayez en majuscule)"
    return 100
  else
    echo "${result}"
  fi
}

function __network_control_build_json_enable_disable_internet__ {
  local -r  current_mode="$1"
  local -ri profile_id="$2"
  local -r  mac_addr_upper_case="${3:-}"
  local -r  override_mode="${current_mode}"

  local macs_array_json=
  if [ -n "${mac_addr_upper_case}" ] ; then
    macs_array_json="[ \"${mac_addr_upper_case}\" ]"
  else
    macs_array_json="$( freebox_network_control_json "${profile_id}" | jq -c '[ .hosts[].l2ident.id ]' )" || return $?
  fi

  jq -c --null-input \
    --arg profile_id "${profile_id}" \
    --arg override_mode "${override_mode}" \
    --arg current_mode "${current_mode}" \
    --argjson macs "${macs_array_json}" \
'{
  "profile_id": $profile_id,
  "next_change": 0,
  "override": true,
  "override_mode": $override_mode,
  "current_mode": $current_mode,
  "macs": $macs
}' || return $?
}

function freebox_network_control_enable_disable_internet_by_profile {
  local -r  enable_disable="$1"
  local -ri profile_id="$2"
  local -r  mac_addr_upper_case="${3:-}"

  local current_mode=
  case "${enable_disable}" in
    'disable'|'denied') current_mode='denied' ;;
    'enable'|'allowed') current_mode='allowed' ;;
    *)
      echofatal "${FUNCNAME[0]}() - Wrong mode value='${enable_disable}'"
      return 100
  esac
  local -r current_mode

  if [ "${profile_id}" -le 0 ] ; then
    echofatal "${FUNCNAME[0]}() - Wrong profile id value='${profile_id}'"
    return 100
  fi

  #PUT /api/v4/parental/filter/4 HTTP/1.1
  # freebox --api-put 'parental/filter/15' '{ "forced_mode": "denied", "forced": true }'

  # PUT /api/v8/
  local json=
  json="$( __network_control_build_json_enable_disable_internet__ "${current_mode}" "${profile_id}" "${mac_addr_upper_case}" )" || return $?
  local -r json

  local result=
  result="$( freebox_api_put_with_result_if_success "network_control/${profile_id}" "${json}" )" || return $?

  jq -S '. | del( .hosts[] // [] | .access_point ) | del ( .hosts[] // [] | .l3connectivities )' <<<"${result}" | jq_normalize || return $?
}

function freebox_network_control_enable_disable_internet_by_mac_addr {
  local -r enable_disable="$1"
  local -r mac_addr_upper_case="$2"

  local -i profile_id=
  profile_id="$( __network_control_get_profile_id_for_mac_address__ "${mac_addr_upper_case}" )" || return $?
  local -ri profile_id

  freebox_network_control_enable_disable_internet_by_profile "${enable_disable}" "${profile_id}" || return $?
}

