#!/bin/bash -e
#
# freebox-tools:bin/freebox.sh
#
set -u
set -o pipefail

PROJECT_HOME="$( dirname "$( dirname "$( realpath "${BASH_SOURCE[0]}" )" )" )"
declare -r PROJECT_HOME

# shellcheck disable=SC1091
source "${PROJECT_HOME}/apis/bash-common-api/bash-common-api.sh" || exit $?
# shellcheck disable=SC1091
source "${PROJECT_HOME}/apis/bash-common-api/json-api.sh" || exit $?
# shellcheck disable=SC1091
source "${PROJECT_HOME}/bin/api/freebox-tools-api.sh" || exit $?
# shellcheck disable=SC1091
source "${PROJECT_HOME}/bin/api/freebox-bash-api.sh" || exit $?
# shellcheck disable=SC1091
source "${PROJECT_HOME}/bin/api/freebox-config-api.sh" || exit $?
# shellcheck disable=SC1091
source "${PROJECT_HOME}/bin/api/freebox-network-api.sh" || exit $?
# shellcheck disable=SC1091
source "${PROJECT_HOME}/bin/api/freebox-audit-api.sh" || exit $?
# shellcheck disable=SC1091
source "${PROJECT_HOME}/bin/api/freebox-plug-api.sh" || exit $?

function usage {
  local full="${1:-false}"

  cat <<EOF
Boite à outils pour votre Freebox. Notez que certaines fonctionnalités demandent des droits
supplémentaires qu’il faut ajouter depuis l’interface de votre Freebox.

Usage:
  $( basename "${BASH_SOURCE[0]}" ) --help

  $( basename "${BASH_SOURCE[0]}" ) --init-token

  $( basename "${BASH_SOURCE[0]}" ) --api
  $( basename "${BASH_SOURCE[0]}" ) --api-text
  $( basename "${BASH_SOURCE[0]}" ) --connection
  $( basename "${BASH_SOURCE[0]}" ) --connection-text
  $( basename "${BASH_SOURCE[0]}" ) --display-hosts
  $( basename "${BASH_SOURCE[0]}" ) --display-hosts-text [--separator SEPARATOR] [--separator LASTSEP]
  $( basename "${BASH_SOURCE[0]}" ) --json URI # Identique à GET
  $( basename "${BASH_SOURCE[0]}" ) --network-disable-by-mac ADDR
  $( basename "${BASH_SOURCE[0]}" ) --network-disable-by-profile ID
  $( basename "${BASH_SOURCE[0]}" ) --network-enable-by-mac ADDR
  $( basename "${BASH_SOURCE[0]}" ) --network-enable-by-profile ID
  $( basename "${BASH_SOURCE[0]}" ) --parental-config
  $( basename "${BASH_SOURCE[0]}" ) --parental-filter ID
  $( basename "${BASH_SOURCE[0]}" ) --parental-filters
  $( basename "${BASH_SOURCE[0]}" ) --parental-filters-id
  $( basename "${BASH_SOURCE[0]}" ) --plug
  $( basename "${BASH_SOURCE[0]}" ) --plug-reset ID
  $( basename "${BASH_SOURCE[0]}" ) --plug-reset-all
  $( basename "${BASH_SOURCE[0]}" ) --profile [ID]
  $( basename "${BASH_SOURCE[0]}" ) --profile-network [ID]
  $( basename "${BASH_SOURCE[0]}" ) --restart
  $( basename "${BASH_SOURCE[0]}" ) --show-mac-address
  $( basename "${BASH_SOURCE[0]}" ) --version

  $( basename "${BASH_SOURCE[0]}" ) get URI
  $( basename "${BASH_SOURCE[0]}" ) post URI [--data DATA]
  $( basename "${BASH_SOURCE[0]}" ) put URI --data DATA
EOF

  if [ "${full}" = 'true' ] ; then
    cat <<EOF

Actions:
  -h,   --help          Affiche ce message d’aide et quitte.

  -I,   --init-token    Configuration: Récupère un nouveau token pour l’application
                        et le sauvegarde.

        --api           Affiche les informations disponibles sur l’API de votre
                        Freebox au format JSON.
        --api-text      Affiche les informations disponibles sur l’API de votre
                        Freebox au format TEXTE.

        --calls,
        --call-log      Retourne la liste des appels au format JSON.

        --call-numbers-text
                        Affiches la liste des numéros de téléphones ayant appelés au
                        format TEXTE (un numéro par ligne).

  -C,   --connection    Affiche certaines informations de connexion de votre Freebox
                        au format JSON.
        --connection-text
                        Affiche certaines informations de connexion de votre Freebox.
                        au format TEXTE.

        --display-hosts Affiche la liste des machines connues au format JSON.
        --display-hosts-text
                        Affiche la liste des machines connues au format TEXTE:
                            Adresse IPv4
                            Adresse IPv6
                            Adresse MAC
                            Type de la machine
                            Constructeur de la machine
                            Nom de la machine

        --hosts         Affiche la liste des machines détaillée connues
                        (raccourci pour GET /lan/browser/pub)

        --parental-config
                        Configuration du contrôle parental au format JSON

        --parental-filter ID
                        Configuration du filtre ID du contrôle parental au format JSON

                        Par exemple:

                        ${BASH_SOURCE[0]} --parental-filter 3

        --parental-filters
                        Configuration de tous les filtres du contrôle parental au format
                        JSON

        --parental-filters-id
                        Affiche le nom et l’id des filtres du contrôle parental au format
                        JSON

  -P,   --plug          Liste de vos boîtiers CPL (freeplugs et autre).
        --plug-reset ID Redémarrage du boîtier CPL donné
        --plug-reset-all
                        Redémarrage de tous les boîtiers CPL trouvés.

        --profile [ID]  Sans paramètre affiche la liste de tous les profils, si ID est
                        un nombre affiche le profil correspondant à l'ID.
                        [Nécessite l'API V8]

        --profile-network [ID]
                        Sans paramètre affiche toutes les configurations, si ID est
                        un nombre affiche le profil réseaux correspondant à l'ID.

  -R,   --restart       Demande un redémarrage de la Freebox.

        --show-mac-address
                        Retourne la liste des adresses MAC connues par votre
                        Freebox au format JSON.
        --show-mac-address-text
                        Retourne la liste des adresses MAC connues par votre
                        Freebox au format TEXTE.

        --network-disable-by-profile ID
        --network-enable-by-profile ID
                        Désactive ou active l’accès internet pour un profile donné,
                        requière l'identificateur du profile.

        --network-disable-by-mac ADDR
        --network-enable-by-mac ADDR
                        Désactive ou active l’accès internet pour un profile donné,
                        requière l'adresse MAC d'une des machines du profile.

        --version       Affiche la version et quitte.

        get URI,
        --api-get URI,
        --json URI      Lance une requête HTTP/GET sur la box et affiche le
                        résultat brut au format JSON. Les éventuels paramètres
                        doivent être correctement encodés.

                        Quelques exemples:

                        Caractéristique de votre ligne
                        ${BASH_SOURCE[0]} get /connection/xdsl

                        Votre configuration DHCP
                        ${BASH_SOURCE[0]} get /dhcp/config

                        Vos règles de délégations (forwarding)
                        ${BASH_SOURCE[0]} get /fw/redir/

                        La configuration du contrôle parental
                        ${BASH_SOURCE[0]} get /parental/config/

        post URI  ,
        --api-post URI
                        Lance une requête HTTP/POST sur la box (voir --data)
                        Affiche le résultat brut au format JSON.

        put URI DATA,
        --api-put URI DATA
                        Lance une requête HTTP/PUT sur la box (voir --data)
                        Affiche le résultat brut au format JSON.

                        Par exemple:

                        ${BASH_SOURCE[0]} put '/parental/filter/2' -d '{"forced_mode": "allowed", "forced": true}'
                        ${BASH_SOURCE[0]} put '/parental/filter/2' -d '{"forced_mode": "denied", "forced": true}'
                        ${BASH_SOURCE[0]} put '/parental/filter/2' -d '{"forced": false}'

                        Exemple qui n'est plus supporté depuis la version 8 de l’API
                        ${BASH_SOURCE[0]} put '/parental/config' -d '{"default_filter_mode": "denied"}

Options:

  -A BOOL,          --active BOOL
                        N'affiche que les machines étant actuellement connectée.

  -F FAMILY,
  -f FAMILY,        --family FAMILY
                        Les interfaces de la famille donnée. FAMILY peut prendre l'une des
                        valeurs suivantes: ipv4 ou ipv6

  -S SEPARATOR,     --separator SEPARATOR
                        Défini le séparateur de colonne (défaut: ${SEPARATOR})

  -L LASTSEP,       --last-separator LASTSEP
                        Défini le séparateur de fin de ligne (défaut: ${END_SEPARATOR})

  -k,               --insecure
                        Autorise les connexions non certifiées lors de l’utilisation de SSL

                    --http,
                    --allow-http
                        Autorise l’utilisation du protocole http (non sécurisé)

  -d DATA,          --data DATA
                        Données à joindre à la requête. La chaîne DATA qui doit être un JSON bien formé.

EOF
  fi
}

function is_required {
  local -r value="$1"
  local -r message="$2"

   if [ -z "${value}" ] ; then
    echofatal "${message}" >&2
    echo >&2
    usage >&2
    exit 1
  fi
}

function main {
  local action=
  local param=
  local param_data=
  local param_id=
  local param_uri=
  local param_value=

  while [ "$#" -gt 0 ] ; do
    param="$1"
    shift

    case "${param}" in
      '-h'|'--help')
        usage 'true'
        exit 0
        ;;

      '--version')
        display_version
        exit 0
        ;;

      '-I'|'--init-token')
        action='--init-token'
        ;;

      '-R'|'--restart')
        action='--restart'
        ;;

      'get'|'GET'|'--api-get'|'--json')
        action='GET'
        param_uri="$( P_param "$@" )" && shift || return $?
        ;;

      'post'|'POST'|'--api-post')
        action='POST'
        param_uri="$( P_param "$@" )" && shift || return $?
        ;;

      'put'|'PUT'|'--api-put')
        action='PUT'
        param_uri="$( P_param "$@" )" && shift || return $?
        ;;

      '-C'|'--connection')
        action='--connection'
        ;;

      '--calls'|'--call-log')
        action='--calls'
        ;;

      '-P'|'--plug')
        action='--plug'
        ;;

      '--parental-filter'|\
      '--plug-reset')
        action="${param}"
        param_id="$( P "$@" )" && shift
        ;;

      '--network-control'|\
      '--profile'|\
      '--profile-network')
        action="${param}"
        if is_positive_integer "${1:-}" ; then
          param_id="$( P "$@" )" && shift
        else
          param_id='0'
        fi
        ;;

      '--network-disable-by-profile'|\
      '--network-enable-by-profile'|\
      '--network-disable-by-mac'|\
      '--network-enable-by-mac')
        action="${param}"
        param_value="$( P_param "$@" )" && shift || return $?
        ;;

      '--api'|'--api-text'|\
      '--call-numbers-text'|\
      '--connection-text'|\
      '--display-hosts'|\
      '--display-hosts-text'|\
      '--hosts'|\
      '--parental-config'|\
      '--parental-filters'|\
      '--parental-filters-id'|\
      '--plug-reset-all'|\
      '--show-mac-address'|\
      '--show-mac-address-text')
        action="${param}"
        ;;

      '-A'|'--active')
        param_active="$( P_param "$@" )" && shift || return $?
        declare -gx param_active
        ;;

      '-F'|'-f'|'--family')
        param_family="$( P_param "$@" )" && shift || return $?
        declare -gx param_family
        ;;

      '-d'|'--data')
        param_data="$( P_param "$@" )" && shift || return $?
        ;;

      '-S'|'--separator')
        SEPARATOR="$( P_param "$@" )" && shift || return $?
        ;;

      '-L'|'--last-separator')
        END_SEPARATOR="$( P_param "$@" )" && shift || return $?
        ;;

      '-k'|'--insecure'         ) set_insecure ;;
      '--http'|'--allow-http'   ) set_allow_http ;;

      *)
        # unknown option
        echofatal "Paramètre inconnu: '${param}'."
        usage >&2
        exit 1
        ;;
    esac
  done

  if [ -z "${action}" ] ; then
    echofatal 'Paramètre manquant.' >&2
    usage >&2
    return 100
  elif [ "${action}" = '--init-token' ] ; then
    freebox_config_init_token || return $?
  else
    freebox_connect || return $?

    case "${action}" in
      '--api'                 ) show_api_json || return $? ;;
      '--api-text'            ) show_api_text || return $? ;;
      '--call-numbers-text'   ) freebox_api_get_with_result_if_success '/call/log/' | jq -r '.[].number' || return $? ;;
      '--calls'               ) freebox_api_get_with_result_if_success '/call/log/' | jq '.' || return $? ;;
      '--connection'          ) show_connection_json || return $? ;;
      '--connection-text'     ) show_connection_text || return $? ;;
      '--display-hosts'       ) freebox_display_hosts_json || return $? ;;
      '--display-hosts-text'  ) freebox_display_hosts_text || return $? ;;
      '--hosts'               ) get_hosts_json || return $? ;;
      '--parental-config'     ) freebox_get_parental_config_json | jq_normalize || return $? ;;
      '--parental-filters'    ) freebox_get_parental_filters_json | jq_normalize || return $? ;;
      '--plug'                ) freebox_api_get '/freeplug' | jq_normalize || return $? ;;
      '--plug-reset-all'      ) freebox_reset_all_plugs || return $? ;;
      '--restart'             ) freebox_reboot || return $? ;;
      '--show-mac-address'        ) freebox_show_mac_addresses_json || return $? ;;
      '--show-mac-address-text'   ) freebox_show_mac_addresses_text || return $? ;;

      '--network-control')
        is_required "${param_id}" "L'action --network-control attend un id (ex: --network-control '1')"
        freebox_network_control_json "${param_id}" || return $?
        ;;

      '--network-disable-by-profile')
        freebox_network_control_enable_disable_internet_by_profile 'disable' "${param_value}" || return $?
        ;;

      '--network-enable-by-profile')
        freebox_network_control_enable_disable_internet_by_profile 'enable' "${param_value}" || return $?
        ;;

      '--network-disable-by-mac')
        freebox_network_control_enable_disable_internet_by_mac_addr 'disable' "${param_value}" || return $?
        ;;

      '--network-enable-by-mac')
        freebox_network_control_enable_disable_internet_by_mac_addr 'enable' "${param_value}" || return $?
        ;;

      '--parental-filter')
        is_required "${param_id}" "L'action --parental-filter attend un id (ex: --parental-filter '1')"
        freebox_get_parental_filter_json "${param_id}" | jq_normalize || return $?
        ;;

      '--parental-filters-id')
        freebox_get_parental_filters_json |
            jq '[ .[] | { id:.id, desc:.desc } ]' | jq_normalize || return $?
        ;;

      '--plug-reset')
        is_required "${param_id}" 'Le paramètre --plug-reset attend un id (ex: --plug-reset "F4:CA:E5:1D:46:AE")'
        freebox_api_post "/freeplug/${param_id}/reset/" | jq '.' || return $?
        ;;

      '--profile')
        if [ "${param_id}" -eq 0 ] ; then
          freebox_api_get '/profile/' | jq '.' || return $?
        else
          freebox_api_get "/profile/${param_id}" | jq '.' || return $?
        fi
        ;;

      '--profile-network')
        if [ "${param_id}" -eq 0 ] ; then
          freebox_api_get '/network_control' | jq '.' || return $?
        else
          freebox_api_get "/network_control/${param_id}" | jq '.' || return $?
        fi
        ;;

      'GET')
        is_required "${param_uri}" "L'action GET attend un suffixe d'API (ex: GET '/lan/browser/interfaces/')"
        freebox_api_get "${param_uri}" | jq '.' || return $?
        ;;

      'POST')
        is_required "${param_uri}" "L'action POST attend un suffixe d'API (ex: POST '/lan/browser/interfaces/')"
        freebox_api_post "${param_uri}" "${param_data}" | jq '.' || return $?
        ;;

      'PUT')
        is_required "${param_uri}" "L'action PUT attend un suffixe d'API (ex: POST '/lan/browser/interfaces/')"
        is_required "${param_data}" "L'action PUT attend des données au format JSON (--data)')"
        freebox_api_put "${param_uri}" "${param_data}" | jq '.' || return $?
        ;;

      *)
        echoerror "(BUG) Action non gérée: '${action}'." >&2
        usage >&2
        return 100
        ;;
    esac
  fi
}

main "$@" || exit $?
